/*******************************************************************
* DivXripper - by Hellstorm                                        *
* licensed under the GNU General Public License (GPL) v2           *
* main.cpp - main file of the program                              *
*******************************************************************/

#include "mpeg2.h"
#include "convert.h"
#include "play.h"
#include "version.h"

#include <unistd.h>
#include <string.h>
#include <stdlib.h>

mpeg3_t *file;
mpeg3_t *file2;
char *fileName;
int astream = 0;
int vstream = 0;
int play_mode = 0;
int show_properties = 0;
float ratio = 16.0 / 9.0;
float cropRatio = -1;
char outputFileName[255];
int thereIsOutputFileName = 0;
int bitrate = 0;
int mp3bitrate = 128;
int dwidth = 0;
int dheight = 0;
int dcrop = -1;
int crisp = 70;
char *filelist[128];
double audiocorrection = 0;

void usage(char *progName)
{
	fprintf(stderr,
		"DivXripper v %s by Hellstorm (helly@he11storm.net) and"
		" Andreas Graf (andreas@csg.dyn.dhs.org)\n",
		VERSION);
	fprintf(stderr, "Usage: %s [options] <movie file(s)>\n", progName);
	fprintf(stderr,
		"Options:\t-a STREAM. Decode STREAM audio stream (default 0)\n");
	fprintf(stderr,
		"\t\t-v STREAM. Decode STREAM video stream (default 0)\n");
	fprintf(stderr, "\t\t-p. Play mode\n");
	//fprintf(stderr,"\t\t-2. Double size in play mode\n");
	fprintf(stderr, "\t\t-s. Show properties of film (default off)\n");
	fprintf(stderr,
		"\t\t-r ratio. Aspect Ratio (default 1.78 i.e. 16:9)\n");
	fprintf(stderr,
		"\t\t-o file. Output file name (default video.out)\n");
	fprintf(stderr,
		"\t\t-d bitrate. DivX ;-) Bitrate (default calculated to fit on 1 CD -700 MByte)\n");
	fprintf(stderr, "\t\t-m bitrate. MP3 Bitrate (default 128)\n");
	fprintf(stderr,
		"\t\t-x output width (default=0=original, -1=original stretched to fit ratio)\n");
	fprintf(stderr,
		"\t\t-y output height (default=0=original clinched to fit ratio, -1=original)\n");
	fprintf(stderr, "\t\t-f crispness in percent (default 70)\n");
	fprintf(stderr,
		"\t\t-l lines-based cropping:output lines to skip between top and image (default=0)\n");
	fprintf(stderr,
		"\t\t-R ratio-based cropping:cropping based on ratio between width and height (default=no cropping)\n");
	fprintf(stderr,
		"\t\t-c audiocorrection:amplification factor of the output audio (default automatically calculated)\n");
}

void parseArgs(int argc, char *argv[])
{
	int option;

	opterr = 0;
	while ((option =
		getopt(argc, argv,
		       "R:f:l:o:q:y:x:m:a:v:d:c:psr:2")) != EOF) {
		switch (option) {
		case 'c':
			audiocorrection = atof(optarg);
			break;
		case 'p':
			play_mode = 1;
			break;
		case 's':
			show_properties = 1;
			break;
		case 'q':
			switch (*optarg) {
			case 'l':
				dwidth = 400;
				dheight = 225;
				break;
			case 'm':
				dwidth = 512;
				dheight = 288;
				break;
			case 'h':
				dwidth = 0;
				dheight = 0;
				break;
			case 'b':
				dwidth = -1;
				dheight = -1;
				break;
			}
			break;
		case 'a':
			astream = atoi(optarg);
			break;
		case 'v':
			vstream = atoi(optarg);
			break;
		case 'r':
			ratio = atof(optarg);
			break;
		case 'd':
			bitrate = atoi(optarg);
			break;
		case 'm':
			mp3bitrate = atoi(optarg);
			break;
		case 'x':
			dwidth = atoi(optarg);
			break;
		case 'y':
			dheight = atoi(optarg);
			break;
		case 'l':
			dcrop = atoi(optarg);
			break;
		case 'R':
			cropRatio = atof(optarg);
			break;
		case 'f':
			crisp = atoi(optarg);
			break;
		case 'o':
			strcpy(outputFileName, optarg);
			thereIsOutputFileName = 1;
			break;
		case '?':
			printf("Unknown option:%c\n", optopt);
			usage(argv[0]);
			exit(1);
			break;
		}

	}
	if (optind == argc) {
		fprintf(stderr, "Not enough arguments.\n"
			"You must enter a movie filename\n");
		usage(argv[0]);
		exit(1);
	}
	char **h = filelist;
	while (optind < argc)
		*h++ = argv[optind++];
	*h = 0;


	if (!thereIsOutputFileName) {
		strcpy(outputFileName, "video.avi");
	}
}

void main(int argc, char *argv[])
{
	int numAStreams, numVStreams;
	int filenr, lastfile;
	char **h;
	int frames;

	parseArgs(argc, argv);

	if ((!bitrate || !audiocorrection) && !play_mode) {
		double videoFrames;
		float frameRate;
		long i;
		filenr = 0;
		lastfile = 0;
		videoFrames = 0;
		h = filelist;
		while (*h) {
			fileName = *h++;
			filenr++;
			if (!(*h))
				lastfile = !0;
			if (mpeg2IsFileCompatible(fileName)) {
				if ((file =
				     mpeg2OpenFile(fileName)) == NULL) {
					fprintf(stderr,
						"%s could not be opened\n",
						fileName);
					exit(1);
				}
			} else {
				fprintf(stderr,
					"%s is not a compatible file\n",
					fileName);
				exit(1);
			}
			numVStreams = mpeg2NumVideoStreams(file);
			if (numVStreams
			    && (vstream < 0 || vstream >= numVStreams)) {
				fprintf(stderr,
					"Wrong selected video stream:%d."
					"\nNum Audio Streams:%d\n",
					vstream,
					mpeg2NumVideoStreams(file));
				exit(1);
			}
			if (filenr == 1) {
				frameRate =
				    mpeg3_frame_rate(file, vstream);
				printf("\nFrame Rate: %f frames/sec\n",
				       frameRate);
			}
			double ac;
			i = gettime(file, vstream, astream, filenr,
				    lastfile, &frames, &ac);
			if (!audiocorrection)
				audiocorrection = ac;
			mpeg2CloseFile(file);
		}
		double fr = 700. * 1024. * 8. * frameRate / (double) frames;	// 700 MByte CD size
		fr -= mp3bitrate;
		bitrate = (int) fr;
		printf
		    ("total movie time: %d s\nSetting video bitrate to %d kbps\n",
		     (int) (frames / frameRate), bitrate);
	}

	filenr = 0;
	lastfile = 0;
	h = filelist;
	while (*h) {

		fileName = *h++;
		filenr++;

		printf
		    ("\n\n***************************************************\nprocessing input file %s, file #%d",
		     fileName, filenr);

		if (!(*h))
			lastfile = !0;

		if (!lastfile)
			printf("\n\n");
		else
			printf(" (last file)\n\n");
		fflush(stdout);

		if (mpeg2IsFileCompatible(fileName)) {
			printf("File is compatible :-)\n");
			if ((file = mpeg2OpenFile(fileName)) == NULL) {
				fprintf(stderr, "%s could not be opened\n",
					fileName);
				exit(1);
			}
			if (play_mode) {
				if ((file2 =
				     mpeg2OpenCopyFile(file,
						       fileName)) ==
				    NULL) {
					fprintf(stderr,
						"%s copy could not be opened\n",
						fileName);
					exit(1);
				}
			}
		} else {
			fprintf(stderr, "%s is not a compatible file\n",
				fileName);
			exit(1);
		}

              setuid(getuid());
              setgid(getgid());

		numAStreams = mpeg2NumAudioStreams(file);
		if (show_properties) {
			mpeg2ShowFileInfo(file);
			mpeg2CloseFile(file);
			exit(0);
		}
		if (numAStreams && (astream < 0 || astream >= numAStreams)) {
			fprintf(stderr, "Wrong selected audio stream:%d."
				"\nNum Audio Streams:%d\n", astream,
				mpeg2NumAudioStreams(file));
			exit(1);
		}
		numVStreams = mpeg2NumVideoStreams(file);
		if (numVStreams && (vstream < 0 || vstream >= numVStreams)) {
			fprintf(stderr, "Wrong selected video stream:%d."
				"\nNum Audio Streams:%d\n", vstream,
				mpeg2NumVideoStreams(file));
			exit(1);
		}

		if (play_mode) {
			playInit(file, file2, astream, vstream, ratio,
				 dcrop, cropRatio);
			mpeg2CloseFile(file2);
		} else {
			convertFile(file, astream, vstream, ratio,
				    outputFileName, (int) (bitrate * 1.00),
				    mp3bitrate, dwidth, dheight, dcrop,
				    cropRatio, crisp, filenr, lastfile,
				    audiocorrection);
		}

		mpeg2CloseFile(file);
	}
}
